@@ -14,11 +14,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
+import logging
 import os
-import sys
-import commands
+import shlex
 import subprocess
+import sys
+import time
+import traceback
+import unittest
+import StringIO
 
+import virtinst.cli
+
+from scriptimports import virtinstall, virtimage, virtclone, virtconvert
 import utils
 
 os.environ["VIRTCONV_TEST_NO_DISK_CONVERSION"] = "1"
@@ -78,6 +86,8 @@ new_files   = new_images + virtimage_new + virtconv_dirs
 clean_files = (new_images + exist_images +
                virtimage_exist + virtimage_new + virtconv_dirs + [ro_dir])
 
+promptlist = []
+
 test_files = {
     'TESTURI'           : testuri,
     'REMOTEURI'         : remoteuri,
@@ -103,6 +113,7 @@ test_files = {
     'TREEDIR'           : treedir,
     'MANAGEDEXIST1'     : "/default-pool/testvol1.img",
     'MANAGEDEXIST2'     : "/default-pool/testvol2.img",
+    'MANAGEDEXISTUPPER' : "/default-pool/UPPER",
     'MANAGEDNEW1'       : "/default-pool/clonevol",
     'MANAGEDNEW2'       : "/default-pool/clonevol",
     'MANAGEDDISKNEW1'   : "/disk-pool/newvol1.img",
@@ -115,66 +126,16 @@ test_files = {
     'VMX_IMG1'          : "%s/vmx/test1.vmx" % vcdir,
 }
 
-debug = False
-
-class PromptCheck(object):
-    def __init__(self, prompt, response=None):
-        self.prompt = prompt
-        self.response = response
-        if self.response:
-            self.response = self.response % test_files
-
-    def check(self, proc):
-        out = proc.stdout.readline()
-
-        if not out.count(self.prompt):
-            out += "\nContent didn't contain prompt '%s'" % (self.prompt)
-            return False, out
-
-        if self.response:
-            proc.stdin.write(self.response + "\n")
-
-        return True, out
-
-class PromptTest(object):
-    def __init__(self, cmd):
-        cmd = cmd % test_files
-        app, opts = cmd.split(" ", 1)
-        self.cmd = [os.path.abspath(app)] + opts.split(" ")
-        self.cmdstr = cmd
-        self.debug = False
-
-        self.prompt_list = []
-
-    def add(self, *args, **kwargs):
-        self.prompt_list.append(PromptCheck(*args, **kwargs))
-
-    def run(self):
-        proc = subprocess.Popen(self.cmd,
-                stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT)
-
-        out = "Running %s\n" % self.cmdstr
-
-        for p in self.prompt_list:
-            ret, content = p.check(proc)
-            out += content
-            if not ret:
-                # Since we didn't match output, process might be hung
-                proc.kill()
-                break
-
-        return proc.wait(), out
 
 """
 CLI test matrix
 
+Any global args for every invocation should be added to default_args
+function, so that individual tests can easily overwrite them.
+
 Format:
 
 "appname" {
-  "globalargs" : Arguments to be passed to every app invocation
-
   "categoryfoo" : { Some descriptive test catagory name (e.g. storage)
 
     "args" : Args to be applied to all invocations in category
@@ -193,12 +154,49 @@ Format:
 }
   "
 """
+
+
+def default_args(app, cli, testtype):
+    args = ""
+    iscompare = testtype in ["compare"]
+
+    if not iscompare:
+        args = "--debug"
+
+    if app in ["virt-install", "virt-clone", "virt-image"] and not iscompare:
+        if "--connect " not in cli:
+            args += " --connect %(TESTURI)s"
+
+    if app in ["virt-install"]:
+        if "--name " not in cli:
+            args += " --name foobar"
+        if "--ram " not in cli:
+            args += " --ram 64"
+
+    if testtype in ["compare"]:
+        if app == "virt-install":
+            if (not cli.count("--print-xml") and
+                not cli.count("--print-step") and
+                not cli.count("--quiet")):
+                args += " --print-step all"
+
+        elif app == "virt-image":
+            if not cli.count("--print"):
+                args += " --print"
+
+        elif app == "virt-clone":
+            if not cli.count("--print-xml"):
+                args += " --print-xml"
+
+        if app != "virt-convert" and not "--connect " in cli:
+            args += " --connect %s" % fakeuri
+
+    return args
+
 args_dict = {
 
 
   "virt-install" : {
-    "globalargs" : " --connect %(TESTURI)s -d --name foobar --ram 64",
-
     "storage" : {
       "args": "--pxe --nographics --noautoconsole --hvm",
 
@@ -570,6 +568,10 @@ args_dict = {
         "--hvm --pxe --filesystem template_name,/,type=template",
         # no networks
         "--hvm --nodisks --nonetworks --cdrom %(EXISTIMG1)s",
+        # --memballoon use virtio
+        "--hvm --nodisks --pxe --memballoon virtio",
+        # --memballoon disabled
+        "--hvm --nodisks --pxe --memballoon none",
       ],
 
       "invalid": [
@@ -589,6 +591,8 @@ args_dict = {
         "--paravirt --import --disk path=virt-install --print-step 2",
         # 2 stage install with --print-xml
         "--hvm --nodisks --pxe --print-xml",
+        # Busted --memballoon
+        "--hvm --nodisks --pxe --memballoon foobar",
       ],
 
       "compare": [
@@ -604,7 +608,7 @@ args_dict = {
          "--controller usb,model=ich9-uhci1,address=0:0:4.0,index=0,master=0 "
          "--controller usb,model=ich9-uhci2,address=0:0:4.1,index=0,master=2 "
          "--controller usb,model=ich9-uhci3,address=0:0:4.2,index=0,master=4 "
-         "--disk %(EXISTIMG1)s,cache=writeback,io=threads,perms=sh,serial=WD-WMAP9A966149 "
+         "--disk %(MANAGEDEXISTUPPER)s,cache=writeback,io=threads,perms=sh,serial=WD-WMAP9A966149 "
          "--disk %(NEWIMG1)s,sparse=false,size=.001,perms=ro,error_policy=enospace "
          "--disk device=cdrom,bus=sata "
          "--serial tcp,host=:2222,mode=bind,protocol=telnet "
@@ -614,7 +618,8 @@ args_dict = {
          "--channel spicevmc "
          "--smartcard passthrough,type=spicevmc "
          "--security type=static,label='system_u:object_r:svirt_image_t:s0:c100,c200',relabel=yes "
-         """ --numatune \\"1-3,5\\",mode=preferred """,
+         """ --numatune \\"1-3,5\\",mode=preferred """
+         "--boot loader=/foo/bar ",
          "many-devices"),
       ],
 
@@ -625,13 +630,13 @@ args_dict = {
 
       "valid": [
         # Just a macaddr
-        "--mac 11:22:33:44:55:AF",
+        "--mac 22:22:33:44:55:AF",
         # user networking
         "--network=user",
         # Old bridge option
         "--bridge mybr0",
         # Old bridge w/ mac
-        "--bridge mybr0 --mac 11:22:33:44:55:AF",
+        "--bridge mybr0 --mac 22:22:33:44:55:AF",
         # --network bridge:
         "--network bridge:mybr0,model=e1000",
         # VirtualNetwork with a random macaddr
@@ -639,11 +644,11 @@ args_dict = {
         # VirtualNetwork with a random macaddr
         "--network network:default --mac 00:11:22:33:44:55",
         # Using '=' as the net type delimiter
-        "--network network=default,mac=11:00:11:00:11:00",
+        "--network network=default,mac=22:00:11:00:11:00",
         # with NIC model
         "--network=user,model=e1000",
         # several networks
-        "--network=network:default,model=e1000 --network=user,model=virtio,mac=11:22:33:44:55:AF",
+        "--network=network:default,model=e1000 --network=user,model=virtio,mac=22:22:33:44:55:AF",
       ],
       "invalid": [
         # Nonexistent network
@@ -653,7 +658,7 @@ args_dict = {
         # Mixing bridge and network
         "--network user --bridge foo0",
         # Colliding macaddr
-        "--mac 11:22:33:12:34:AB",
+        "--mac 22:22:33:12:34:AB",
       ],
 
      }, # category "network"
@@ -851,8 +856,6 @@ args_dict = {
 
 
   "virt-clone": {
-    "globalargs" : " --connect %(TESTURI)s -d",
-
     "general" : {
       "args": "-n clonetest",
 
@@ -931,7 +934,7 @@ args_dict = {
       ],
 
       "compare" : [
-        ("-o test-for-clone --auto-clone --clone-running", "clone-auto1"),
+        ("--connect %(KVMURI)s -o test-for-clone --auto-clone --clone-running", "clone-auto1"),
         ("-o test-clone-simple --name newvm --auto-clone --clone-running",
          "clone-auto2"),
       ],
@@ -959,8 +962,6 @@ args_dict = {
 
 
   'virt-image': {
-    "globalargs" : " --connect %(TESTURI)s --debug",
-
     "general" : {
       "args" : "--name test-image %(IMAGE_XML)s",
 
@@ -1054,8 +1055,6 @@ args_dict = {
 
 
   "virt-convert" : {
-    "globalargs" : "--debug",
-
     "misc" : {
      "args": "",
 
@@ -1094,7 +1093,185 @@ args_dict = {
   }, # app 'virt-convert'
 }
 
-promptlist = []
+_conns = {}
+def open_conn(uri):
+    #if uri not in _conns:
+    #    _conns[uri] = virtinst.cli.getConnection(uri)
+    #return _conns[uri]
+    return virtinst.cli.getConnection(uri)
+
+######################
+# Test class helpers #
+######################
+
+class Command(object):
+    """
+    Instance of a single cli command to test
+    """
+    def __init__(self, cmd):
+        self.cmdstr = cmd % test_files
+        self.check_success = True
+        self.compare_file = None
+
+        app, opts = self.cmdstr.split(" ", 1)
+        self.argv = [os.path.abspath(app)] + shlex.split(opts)
+
+    def _launch_command(self):
+        logging.debug(self.cmdstr)
+
+        uri = None
+        conn = None
+        app = self.argv[0]
+
+        for idx in reversed(range(len(self.argv))):
+            if self.argv[idx] == "--connect":
+                uri = self.argv[idx + 1]
+                break
+
+        if uri:
+            conn = open_conn(uri)
+
+        oldstdout = sys.stdout
+        oldstderr = sys.stderr
+        oldargv = sys.argv
+        try:
+            out = StringIO.StringIO()
+            sys.stdout = out
+            sys.stderr = out
+            sys.argv = self.argv
+
+            try:
+                if app.count("virt-install"):
+                    ret = virtinstall.main(conn=conn)
+                elif app.count("virt-clone"):
+                    ret = virtclone.main(conn=conn)
+                elif app.count("virt-image"):
+                    ret = virtimage.main(conn=conn)
+                elif app.count("virt-convert"):
+                    ret = virtconvert.main()
+            except SystemExit, sys_e:
+                ret = sys_e.code
+
+            if ret != 0:
+                ret = -1
+            outt = out.getvalue()
+            if outt.endswith("\n"):
+                outt = outt[:-1]
+            return (ret, outt)
+        finally:
+            sys.stdout = oldstdout
+            sys.stderr = oldstderr
+            sys.argv = oldargv
+
+
+    def _get_output(self):
+        try:
+            for i in new_files:
+                os.system("rm %s > /dev/null 2>&1" % i)
+
+            code, output = self._launch_command()
+
+            logging.debug(output + "\n")
+            return code, output
+        except Exception, e:
+            return (-1, "".join(traceback.format_exc()) + str(e))
+
+    def run(self):
+        filename = self.compare_file
+        err = None
+
+        try:
+            code, output = self._get_output()
+
+            if bool(code) == self.check_success:
+                raise AssertionError(
+                    ("Expected command to %s, but failed.\n" %
+                     (self.check_success and "pass" or "fail")) +
+                     ("Command was: %s\n" % self.cmdstr) +
+                     ("Error code : %d\n" % code) +
+                     ("Output was:\n%s" % output))
+
+            if filename:
+                # Generate test files that don't exist yet
+                if not os.path.exists(filename):
+                    file(filename, "w").write(output)
+
+                utils.diff_compare(output, filename)
+
+        except AssertionError, e:
+            err = self.cmdstr + "\n" + str(e)
+
+        return err
+
+
+class PromptCheck(object):
+    """
+    Individual question/response pair for automated --prompt tests
+    """
+    def __init__(self, prompt, response=None):
+        self.prompt = prompt
+        self.response = response
+        if self.response:
+            self.response = self.response % test_files
+
+    def check(self, proc):
+        out = proc.stdout.readline()
+
+        if not out.count(self.prompt):
+            out += "\nContent didn't contain prompt '%s'" % (self.prompt)
+            return False, out
+
+        if self.response:
+            proc.stdin.write(self.response + "\n")
+
+        return True, out
+
+
+class PromptTest(Command):
+    """
+    Fully automated --prompt test
+    """
+    def __init__(self, cmdstr):
+        Command.__init__(self, cmdstr)
+
+        self.prompt_list = []
+
+    def add(self, *args, **kwargs):
+        self.prompt_list.append(PromptCheck(*args, **kwargs))
+
+    def _launch_command(self):
+        proc = subprocess.Popen(self.argv,
+                stdin=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                stderr=subprocess.STDOUT)
+
+        out = "Running %s\n" % self.cmdstr
+
+        for p in self.prompt_list:
+            ret, content = p.check(proc)
+            out += content
+            if not ret:
+                # Since we didn't match output, process might be hung
+                proc.kill()
+                break
+
+        exited = False
+        for ignore in range(30):
+            if proc.poll() is not None:
+                exited = True
+                break
+            time.sleep(.1)
+
+        if not exited:
+            proc.kill()
+            out += "\nProcess was killed by test harness"
+
+        return proc.wait(), out
+
+
+##########################
+# Automated prompt tests #
+##########################
 
 # Basic virt-install prompting
 p1 = PromptTest("virt-install --connect %(TESTURI)s --prompt --quiet "
@@ -1145,190 +1322,64 @@ p6.add("use as the cloned disk", "%(NEWIMG1)s")
 p6.add("use as the cloned disk", "%(NEWIMG2)s")
 promptlist.append(p6)
 
-def runcomm(comm):
-    try:
-        for i in new_files:
-            os.system("rm %s > /dev/null 2>&1" % i)
+# Basic virt-clone prompting with disk failure handling
+p7 = PromptTest("virt-clone --connect %(TESTURI)s --prompt --quiet "
+               "--clone-running -o test-clone-simple -n test-clone-new")
+p7.add("use as the cloned disk", "/root")
+p7.add("'/root' must be a file or a device")
+p7.add("use as the cloned disk", "%(MANAGEDNEW1)s")
+promptlist.append(p7)
 
-        if type(comm) is str:
-            if debug:
-                print comm % test_files
 
-            code, output = commands.getstatusoutput(comm % test_files)
+#########################
+# Test runner functions #
+#########################
 
-        else:
-            if debug:
-                print comm.cmdstr
-            code, output = comm.run()
-
-        if debug:
-            print output
-            print "\n"
-
-        return code, output
-    except Exception, e:
-        return (-1, str(e))
-
-def write_pass():
-    if not debug:
-        sys.stdout.write(".")
-        sys.stdout.flush()
-
-def write_fail():
-    if not debug:
-        sys.stdout.write("F")
-        sys.stdout.flush()
-
-class Command(object):
-    def __init__(self, cmd):
-        self.cmdstr = cmd
-        self.cmd = None
-        self.check_success = True
-        self.compare_file = None
-
-        if type(cmd) is not str:
-            self.cmd = cmd
-            self.cmdstr = " ".join(self.cmd.cmd)
-
-    def run(self):
-        filename = self.compare_file
-        err = None
-
-        try:
-            code, output = runcomm(self.cmd or self.cmdstr)
-
-            if bool(code) == self.check_success:
-                raise AssertionError(
-                    ("Expected command to %s, but failed.\n" %
-                     (self.check_success and "pass" or "fail")) +
-                     ("Command was: %s\n" % self.cmdstr) +
-                     ("Error code : %d\n" % code) +
-                     ("Output was:\n%s" % output))
-
-            if filename:
-                # Uncomment to generate new test files
-                if not os.path.exists(filename):
-                    file(filename, "w").write(output)
-
-                utils.diff_compare(output, filename)
-
-            write_pass()
-        except AssertionError, e:
-            write_fail()
-            err = self.cmdstr + "\n" + str(e)
-
-        return err
-
-# Setup: build cliarg dict, which uses
-def run_tests(do_app, do_category, error_ret):
-    if do_app and do_app not in args_dict.keys():
-        raise ValueError("Unknown app '%s'" % do_app)
-
-    cmdlist = []
-
-    # Prompt tests upfront
-    for cmd in promptlist:
-        cmdlist.append(Command(cmd))
+def build_cmd_list():
+    cmdlist = promptlist
 
     for app in args_dict:
-        if do_app and app != do_app:
-            continue
-
         unique = {}
-        globalargs = ""
 
         # Build default command line dict
         for option in args_dict.get(app):
-            if option == "globalargs":
-                globalargs = args_dict[app][option]
-                continue
-
             # Default is a unique cmd string
             unique[option] = args_dict[app][option]
 
-
-        if do_category and do_category not in unique.keys():
-            raise ValueError("Unknown category %s" % do_category)
-
         # Build up unique command line cases
         for category in unique.keys():
-            if do_category and category != do_category:
-                continue
             catdict = unique[category]
             category_args = catdict["args"]
 
-            for optstr in catdict["valid"]:
-                cmdstr = "./%s %s %s %s" % (app, globalargs,
-                                            category_args, optstr)
-                cmd = Command(cmdstr)
-                cmd.check_success = True
-                cmdlist.append(cmd)
+            for testtype in ["valid", "invalid", "compare"]:
+                for optstr in catdict.get(testtype) or []:
+                    if testtype == "compare":
+                        optstr, filename = optstr
+                        filename = "%s/%s.xml" % (compare_xmldir, filename)
 
-            for optstr in catdict["invalid"]:
-                cmdstr = "./%s %s %s %s" % (app, globalargs,
-                                            category_args, optstr)
-                cmd = Command(cmdstr)
-                cmd.check_success = False
-                cmdlist.append(cmd)
+                    args = category_args + " " + optstr
+                    args = default_args(app, args, testtype) + " " + args
+                    cmdstr = "./%s %s" % (app, args)
 
-            for optstr, filename in catdict.get("compare") or []:
-                filename = "%s/%s.xml" % (compare_xmldir, filename)
-                cmdstr = "./%s %s %s %s" % (app, globalargs,
-                                            category_args, optstr)
-                cmdstr = cmdstr % test_files
+                    cmd = Command(cmdstr)
+                    if testtype == "compare":
+                        cmd.check_success = not filename.endswith("fail.xml")
+                        cmd.compare_file = filename
+                    else:
+                        cmd.check_success = bool(testtype == "valid")
 
-                # Strip --debug to get reasonable output
-                cmdstr = cmdstr.replace("--debug ", "").replace("-d ", "")
-                if app == "virt-install":
-                    if (not cmdstr.count("--print-xml") and
-                        not cmdstr.count("--print-step") and
-                        not cmdstr.count("--quiet")):
-                        cmdstr += " --print-step all"
+                    cmdlist.append(cmd)
 
-                elif app == "virt-image":
-                    if not cmdstr.count("--print"):
-                        cmdstr += " --print"
+    return cmdlist
 
-                elif app == "virt-clone":
-                    if not cmdstr.count("--print-xml"):
-                        cmdstr += " --print-xml"
-
-                if app != "virt-convert" and not cmdstr.count(fakeuri):
-                    cmdstr += " --connect %s" % fakeuri
-
-                cmd = Command(cmdstr)
-                cmd.check_success = not filename.endswith("fail.xml")
-                cmd.compare_file = filename
-                cmdlist.append(cmd)
-
-    # Run commands
-    for cmd in cmdlist:
-        err = cmd.run()
-        if err:
-            error_ret.append(err)
-
-def main():
-    # CLI Args
-    global debug
-
-    do_app = None
-    do_category = None
-
-    if len(sys.argv) > 1:
-        for i in range(1, len(sys.argv)):
-            if sys.argv[i].count("debug"):
-                debug = True
-            elif sys.argv[i].count("--app"):
-                do_app = sys.argv[i + 1]
-            elif sys.argv[i].count("--category"):
-                do_category = sys.argv[i + 1]
-
-    # Setup needed files
-    for i in exist_files:
-        if os.path.exists(i):
-            raise ValueError("'%s' will be used by testsuite, can not already"
-                             " exist." % i)
+newidx = 0
+curtest = 0
+old_bridge = virtinst._util.default_bridge2
 
+def setup():
+    """
+    Create initial test files/dirs
+    """
     os.system("mkdir %s" % ro_dir)
 
     for i in exist_files:
@@ -1338,27 +1389,43 @@ def main():
     os.system("chmod 444 %s" % ro_img)
     os.system("chmod 555 %s" % ro_dir)
 
-    error_ret = []
-    try:
-        run_tests(do_app, do_category, error_ret)
-    finally:
-        cleanup()
-        for err in error_ret:
-            print err + "\n\n"
+    virtinst._util.default_bridge2 = lambda ignore: None
 
-    if not error_ret:
-        print "\nAll tests completed successfully."
 
 def cleanup():
-    # Cleanup files
+    """
+    Cleanup temporary files used for testing
+    """
     for i in clean_files:
         os.system("chmod 777 %s > /dev/null 2>&1" % i)
         os.system("rm -rf %s > /dev/null 2>&1" % i)
 
-if __name__ == "__main__":
-    try:
-        main()
-    except KeyboardInterrupt:
-        print "Tests interrupted"
-        if debug:
-            raise
+    virtinst._util.default_bridge2 = old_bridge
+
+class CLITests(unittest.TestCase):
+    def __init__(self, *args, **kwargs):
+        unittest.TestCase.__init__(self, *args, **kwargs)
+
+    def setUp(self):
+        global curtest
+        curtest += 1
+        # Only run this for first test
+        if curtest == 1:
+            setup()
+
+    def tearDown(self):
+        # Only run this on the last test
+        if curtest == newidx:
+            cleanup()
+
+def maketest(cmd):
+    def cmdtemplate(self, c):
+        err = c.run()
+        if err:
+            self.fail(err)
+    return lambda s: cmdtemplate(s, cmd)
+
+_cmdlist = build_cmd_list()
+for _cmd in _cmdlist:
+    newidx += 1
+    setattr(CLITests, "testCLI%d" % newidx, maketest(_cmd))
