@@ -153,12 +153,7 @@ class CloneDesign(object):
         self._valid_guest        = Guest.Guest(conn=conn)
 
         # Generate a random UUID at the start
-        while 1:
-            uuid = _util.uuidToString(_util.randomUUID())
-            if _util.vm_uuid_collision(self._hyper_conn, uuid):
-                continue
-            self.clone_uuid = uuid
-            break
+        self.clone_uuid = _util.generate_uuid(conn)
 
 
     # Getter/Setter methods
@@ -446,6 +441,11 @@ class CloneDesign(object):
         self._guest.name = self._clone_name
         self._guest.uuid = self._clone_uuid
         self._clone_mac.reverse()
+        for dev in self._guest.get_devices("graphics"):
+            if dev.port and dev.port != -1:
+                logging.warn(_("Setting the graphics device port to autoport, "
+                               "in order to avoid conflicting."))
+                dev.port = -1
         for iface in self._guest.get_devices("interface"):
             iface.target_dev = None
 
@@ -453,7 +453,8 @@ class CloneDesign(object):
                 mac = self._clone_mac.pop()
             else:
                 while 1:
-                    mac = _util.randomMAC(self.original_conn.getType().lower())
+                    mac = _util.randomMAC(self.original_conn.getType().lower(),
+                                          conn=self.original_conn)
                     dummy, msg = self._check_mac(mac)
                     if msg is not None:
                         continue
@@ -477,11 +478,6 @@ class CloneDesign(object):
                     _("Clone onto existing storage volume is not "
                       "supported: '%s'") % clone_disk.path)
 
-            # Change the XML
-            xmldisk.path = None
-            xmldisk.type = clone_disk.type
-            xmldisk.path = clone_disk.path
-
             # Sync 'size' between the two
             if orig_disk.size:
                 clone_disk.size = orig_disk.size
@@ -504,8 +500,15 @@ class CloneDesign(object):
             elif not self.preserve_dest_disks:
                 clone_disk.clone_path = orig_disk.path
 
+            # Change the XML
+            xmldisk.path = None
+            xmldisk.type = clone_disk.type
+            xmldisk.path = clone_disk.path
+            xmldisk.driver_type = orig_disk.driver_type
+
         # Save altered clone xml
         self._clone_xml = self._guest.get_xml_config()
+        logging.debug("Clone guest xml is\n%s", self._clone_xml)
 
     def setup(self):
         """
@@ -513,10 +516,7 @@ class CloneDesign(object):
         additional debug logging.
         """
         self.setup_original()
-        logging.debug("Original guest xml is\n%s", self._original_xml)
-
         self.setup_clone()
-        logging.debug("Clone guest xml is\n%s", self._clone_xml)
 
     def remove_original_vm(self, force=None):
         return self._valid_guest.remove_original_vm(force=force)
@@ -556,7 +556,8 @@ class CloneDesign(object):
                     device = VirtualDisk.DEVICE_CDROM
 
                 d = VirtualDisk(disk.path, conn=self._hyper_conn,
-                                device=device, validate=validate)
+                                device=device, driverType=disk.driver_type,
+                                validate=validate)
                 d.target = disk.target
             except Exception, e:
                 logging.debug("", exc_info=True)
@@ -648,5 +649,5 @@ def _do_duplicate(design, meter):
             logging.debug("Source and destination are the same. Skipping.")
             continue
 
-        # VirtualDisk.setup handles everything
-        dst_dev.setup(meter)
+        # VirtualDisk.setup_dev handles everything
+        dst_dev.setup_dev(meter=meter)
