@@ -255,6 +255,8 @@ def _open_test_uri(uri):
 
     # Need tmpfile names to be deterministic
     if "predictable" in opts:
+        setattr(conn, "_virtinst__fake_conn_predictable", True)
+
         def fakemkstemp(prefix, *args, **kwargs):
             ignore = args
             ignore = kwargs
@@ -262,12 +264,9 @@ def _open_test_uri(uri):
             return os.open(filename, os.O_RDWR | os.O_CREAT), filename
         tempfile.mkstemp = fakemkstemp
 
-        _util.randomMAC = lambda type_: "00:11:22:33:44:55"
-        _util.uuidToString = lambda r: "00000000-1111-2222-3333-444444444444"
-
     # Fake remote status
     if "remote" in opts:
-        _util.is_uri_remote = lambda uri_: True
+        setattr(conn, "_virtinst__fake_conn_remote", True)
 
     # Fake capabilities
     if "caps" in opts:
@@ -437,7 +436,9 @@ def fail(msg, do_exit=True):
     Convenience function when failing in cli app
     """
     logging.error(msg)
-    logging.debug("", exc_info=True)
+    import traceback
+    if traceback.format_exc().strip() != "None":
+        logging.debug("", exc_info=True)
     if do_exit:
         _fail_exit()
 
@@ -707,7 +708,8 @@ def disk_prompt(conn, origpath, origsize, origsparse,
         # If we fail within the loop, reprompt for size and path
         if not retry_path:
             origpath = None
-            origsize = None
+            if not path_to_clone:
+                origsize = None
         retry_path = False
 
         # Get disk path
@@ -1017,6 +1019,16 @@ def get_redirdev(guest, sc_opts):
         if dev:
             guest.add_device(dev)
 
+def get_memballoon(guest, sc_opts):
+    for sc in listify(sc_opts):
+        try:
+            dev = parse_memballoon(guest, sc)
+        except Exception, e:
+            fail(_("Error in memballoon device parameters: %s") % str(e))
+
+        if dev:
+            guest.add_device(dev)
+
 #############################
 # Common CLI option/group   #
 #############################
@@ -1112,6 +1124,9 @@ def add_device_options(devg):
     devg.add_option("", "--redirdev", dest="redirdev", action="append",
                     help=_("Configure a guest redirection device. Ex:\n"
                            "--redirdev usb,type=tcp,server=192.168.1.1:4000"))
+    devg.add_option("", "--memballoon", dest="memballoon", action="append",
+                    help=_("Configure a guest memballoon device. Ex:\n"
+                           "--memballoon medel=virtio"))
 
 def add_gfx_option(devg):
     devg.add_option("", "--graphics", dest="graphics", action="append",
@@ -1355,7 +1370,10 @@ def parse_boot(guest, optstring):
         if val == None:
             return
 
-        setattr(guest.installer.bootconfig, paramname, val)
+        if paramname == "loader":
+            guest.installer.loader = val
+        else:
+            setattr(guest.installer.bootconfig, paramname, val)
 
     # Convert menu= value
     if "menu" in opts:
@@ -1375,6 +1393,7 @@ def parse_boot(guest, optstring):
     set_param("enable_bootmenu", "menu", menu)
     set_param("kernel", "kernel")
     set_param("initrd", "initrd")
+    set_param("loader", "loader")
     set_param("kernel_args", ["kernel_args", "extra_args"])
 
     # Build boot order
@@ -1830,6 +1849,27 @@ def parse_watchdog(guest, optstring, dev=None):
 
     return dev
 
+########################
+# --memballoon parsing #
+########################
+
+def parse_memballoon(guest, optstring, dev=None):
+    if optstring is None:
+        return None
+
+    # Peel the mode off the front
+    opts = parse_optstr(optstring, remove_first="model")
+    model = get_opt_param(opts, "model")
+
+    if not dev:
+        dev = virtinst.VirtualMemballoon(model=model,
+                                         conn=guest.conn)
+
+    if opts:
+        raise ValueError(_("Unknown options %s") % opts.keys())
+
+    return dev
+
 
 ######################################################
 # --serial, --parallel, --channel, --console parsing #
